from xml.etree.ElementTree import Element, SubElement, tostring
from xml.dom import minidom




class Screen(object):
    def __init__(self, name, f_name=None):
        self.f_name = f_name
        self.root = Element('display', version='2.0.0')
        name_child = SubElement(self.root, 'name')
        name_child.text = name

        print (self.prettify(self.root))

    def write(self, elem):
        rough_string = tostring(elem, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        if self.f_name == None:
            return False
        else:
            with open(self.f_name, 'w') as f:
                reparse_xml.writexml(f, indent="  ", addindent="  ", newl="\n", encoding="UTF-8")
            return True

    def prettify(self, elem):
        """Return a pretty-printed XML string for the Element.
        """
        rough_string = tostring(elem, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        return reparse_xml.toprettyxml(indent="  ", newl="\n")




if __name__ == '__main__':
    s = Screen('DisplayTest')



from xml.etree.ElementTree import Element, SubElement, tostring
from argparse import ArgumentParser


class Phoebus(object):
    def __init__(self):
        pass



if __name__ == '__main__':
    p = Phoebus()



""" Phoebus XML Generator

This package contains sub-packages:

test_widget -- Creates individual Phoebus "test_widget"

screen -- Builds complete Phoebus screens from test_widget

"""

__all__ = ["widget", "screen"]


class PVName(object):
    def add_pv_name(self, name):
        self.prop_factory.add_pv_name(name)


class Font(object):
    def add_predefined_font(self, name):
        self.prop_factory.add_font(None, None, None, name)

    def add_font(self, family=None, style=None, size=None):
        self.prop_factory.add_font(family, style, size, None)


class ForegroundColor(object):
    def add_predefined_foreground_color(self, name):
        self.prop_factory.add_foreground_color(name, None, None, None, None)

    def add_foreground_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_foreground_color(None, red, green, blue, alpha)


class BackgroundColor(object):
    def add_predefined_background_color(self, name):
        self.prop_factory.add_background_color(name, None, None, None, None)

    def add_background_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_background_color(None, red, green, blue, alpha)


class Transparent(object):
    def add_transparent(self, transparent=False):
        self.prop_factory.add_transparent(transparent)


class Format(object):
    def add_format(self, format):
        self.prop_factory.add_format(format)


class Precision(object):
    def add_precision(self, val):
        self.prop_factory.add_precision(val)

class ShowUnits(object):
    def add_show_units(self, show=True):
        self.prop_factory.add_show_units(show)


class HorizontalAlignment(object):
    def add_horizontal_alignment(self, val):
        self.prop_factory.add_horizontal_alignment(val)


class VerticalAlignment(object):
    def add_vertical_alignment(self, val):
        self.prop_factory.add_vertical_alignment(val)


class WrapWords(object):
    def add_wrap_words(self, wrap=True):
        self.prop_factory.add_wrap_words(wrap)


class Text(object):
    def add_text(self, text):
        self.prop_factory.add_text(text)


class AutoSize(object):
    def add_auto_size(self, auto=True):
        self.prop_factory.add_auto_size(auto)


# 0, 90, 180, -90
# text update, label, action button
class RotationStep(object):
    def add_rotation_step(self, rotation):
        self.prop_factory.add_rotation_step(rotation)


# picture, symbol
class Rotation(object):
    pass


class Border(object):
    def add_border_width(self, width):
        self.prop_factory.add_border_width(width)

    def add_predefined_border_color(self, name):
        self.prop_factory.add_border_color(name, None, None, None, None)

    def add_border_color(self, red, green, blue, alpha=255):
        self.prop_factory.add_border_color(None, red, green, blue, alpha)



from xml.etree.ElementTree import Element, SubElement
import yaml
import os


# TO DO : Add check if property is already available
# TO DO : Add way to specify defaults in a config file
# TO DO : Default property value should delete element (can use remove element with self.root)
# TO DO : Probably should allow for 'array' input functions to also take in integer value
class Property(object):
    def __init__(self, root_element):
        self.root = root_element
        curr_path = os.path.dirname(__file__)
        with open(curr_path + '/colors.yaml') as f:
            self.predefined_colors = yaml.safe_load(f)
        with open(curr_path + '/fonts.yaml') as f:
            self.predefined_fonts = yaml.safe_load(f)
        self.rotation_steps_array = [0, 90, 180, -90]
        self.horizontal_alignment_array = ['left', 'center', 'right']
        self.vertical_alignment_array = ['top', 'middle', 'bottom']
        self.formats_array = ['default', 'decimal', 'exponential', 'engineering', 'hexadecimal',
                              'compact', 'string',  'sexagesimal hh:mm:ss', 'sexagesimal hms 24h rad',
                              'sexagesimal dms 360deg rad', 'binary']

    def generic_property(self, prop_type, val=None):
        self.root.append(self.create_element(prop_type, val))

    def create_element(self, prop_type, val=None):
        element = Element(prop_type)
        if val is not None:
            if type(val) == bool:
                element.text = str(val).lower()
            else:
                element.text = str(val)
        return element

    def add_pv_name(self, name):
        self.generic_property('pv_name', name)

    def add_precision(self, val):
        try:
            v = int(val)
        except ValueError:
            print('Precision must be an integer, not: {}'.format(val))
            return
        self.generic_property('precision', v)

    def add_show_units(self, show):
        self.generic_property('show_units', show)

    def add_wrap_words(self, wrap):
        self.generic_property('wrap_words', wrap)

    def add_transparent(self, transparent):
        self.generic_property('transparent', transparent)

    def add_rotation(self, rotation):
        pass

    def add_rotation_step(self, rotation):
        try:
            value = int(rotation)
        except ValueError:
            print('Rotation step must be an integer, not: {}'.format(value))
            return
        try:
            val = self.rotation_steps_array.index(value)
        except ValueError:
            print('Invalid rotation given. Must be 0, 90, 180, -90')
            return
        self.generic_property('rotation_step', val)

    def add_horizontal_alignment(self, alignment):
        val = str(alignment).lower()
        try:
            v = self.horizontal_alignment_array.index(val)
        except ValueError:
            print('Wrong input to horizontal alignment: {}. Must be Left, Center, or Right'.format(val))
            return
        self.generic_property('horizontal_alignment', v)

    def add_vertical_alignment(self, alignment):
        val = str(alignment).lower()
        try:
            v = self.vertical_alignment_array.index(val)
        except ValueError:
            print('Wrong input to vertical alignment: {}. Must be Top, Middle, or Bottom'.format(val))
            return
        self.generic_property('vertical_alignment', v)

    # TO DO : limit RGB to 0-255
    def create_color_element(self, root_color_elem, name, red, green, blue, alpha):
        sub_e = self.create_element('color')
        if name is None:
            sub_e.attrib = {'red': str(red), 'blue': str(blue), 'green': str(green), 'alpha': str(alpha)}
        else:
            color_list = self.predefined_colors.get(name.lower())
            if color_list is None:
                print('Color name is undefined')
                return
            else:
                sub_e.attrib = color_list
                sub_e.attrib['name'] = name
        root_color_elem.append(sub_e)
        self.root.append(root_color_elem)

    def add_background_color(self, name, red, green, blue, alpha):
        e = self.create_element('background_color')
        self.create_color_element(e, name, red, green, blue, alpha)

    def add_foreground_color(self, name, red, green, blue, alpha):
        e = self.create_element('foreground_color')
        self.create_color_element(e, name, red, green, blue, alpha)

    # TO DO: Define possible fonts, possible sizes (if applicable), possible styles
    def add_font(self, family, style, size, name):
        e = self.create_element('font')
        if name is not None:
            font_attrib = self.predefined_fonts.get(name.lower())
            if font_attrib is None:
                print('Font name is wrong')
                return
        else:
            font_attrib = {'family': 'Liberation Sans' if family is None else family,
                           'style': 'Regular' if style is None else style,
                           'size': '14' if size is None else str(size)}
        SubElement(e, 'font', attrib=font_attrib)
        self.root.append(e)

    def add_border_width(self, width):
        try:
            v = int(width)
        except ValueError:
            print('Width must be an integer, not: {}'.format(width))
            return
        self.generic_property('border_width', v)

    def add_border_color(self, name, red, green, blue, alpha):
        e = self.create_element('border_color')
        self.create_color_element(e, name, red, green, blue, alpha)

    def add_format(self, format_val):
        val = str(format_val).lower()
        try:
            v = self.formats_array.index(val)
        except ValueError:
            print('Invalid format. Given format {}'.format(format))
            return
        self.generic_property('format', v)

    def add_text(self, text):
        self.generic_property('text', text)

    def add_auto_size(self, auto):
        self.generic_property('auto_size', auto)



from xml.etree.ElementTree import Element, SubElement, tostring
from widget import Widget
from _property_stubs import *


class Label(Widget, Text, Font, ForegroundColor, BackgroundColor, Transparent, HorizontalAlignment,
            RotationStep, WrapWords, AutoSize, Border):
    def __init__(self, name, x, y, width, height, text):
        super().__init__('label', name, x, y, width, height)

        self.add_text(text)


class TextUpdate(Widget, PVName, Font, ForegroundColor, BackgroundColor, Transparent,
                 Format, Precision, ShowUnits, HorizontalAlignment, VerticalAlignment, WrapWords,
                 RotationStep, Border):
    def __init__(self, name, pv_name, x, y, width, height):
        Widget.__init__(self, 'textupdate', name, x, y, width, height)
        self.add_pv_name(pv_name)


if __name__ == '__main__':
    pass



from xml.etree.ElementTree import Element, SubElement, tostring
from xml.dom import minidom
from _property import Property


# TO DO : move macros out of widget class
# TO DO : Add rest of global properties - class, tool tip, actions, rules, scripts
class Widget(object):
    def __init__(self, w_type, name, x_pos, y_pos, width, height, macro_list=None):
        self.root = Element('widget', type=w_type, version='2.0.0')
        name_child = SubElement(self.root, 'name')
        name_child.text = name

        x_child = SubElement(self.root, 'x')
        x_child.text = str(x_pos)

        y_child = SubElement(self.root, 'y')
        y_child.text = str(y_pos)

        width_child = SubElement(self.root, 'width')
        width_child.text = str(width)

        height_child = SubElement(self.root, 'height')
        height_child.text = str(height)

        self.prop_factory = Property(self.root)

        if macro_list is None:
            self.macro_elem = None
        else:
            for m in macro_list:
                self.add_macro(m['name'], m['val'])

    def set_visible(self, visible):
        child = SubElement(self.root, 'visible')
        child.text = str(visible)

    def add_macro(self, name, val):
        if self.macro_elem is None:
            self.macro_elem = SubElement(self.root, 'macros')
        macro = SubElement(self.macro_elem, name)
        macro.text = val

    def find_element(self, tag):
        element = self.root.find(tag)
        return element

    def remove_element(self, tag):
        element = self.find_element(tag)
        if element is not None:
            self.root.remove(element)

    # From: https://pymotw.com/3/xml.etree.ElementTree/create.html
    def prettify(self, elem):
        """Return a pretty-printed XML string for the Element.
        """
        rough_string = tostring(elem, 'utf-8')
        reparse_xml = minidom.parseString(rough_string)
        return reparse_xml.toprettyxml(indent="  ", newl="\n")

    def print_widget(self):
        print(self.prettify(self.root))


if __name__ == '__main__':
    pass
